import { Component, OnDestroy, OnInit } from '@angular/core';
import { MenuItem, PrimeNGConfig } from 'primeng/api';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { TranslateService } from '@ngx-translate/core';
import { GuiService } from 'src/app/service/gui.service';
import { User } from 'src/app/model/auth/user';
import { AuthService } from 'src/app/service/auth.service';
import { MenuElement } from 'src/app/model/view/menu-element';

@Component({
  selector: 'app-header',
  templateUrl: './header.component.html',
  styleUrls: ['./header.component.scss'],
})
export class HeaderComponent implements OnInit, OnDestroy {
  langList: MenuItem[] = [];
  userMenuList!: MenuItem[];
  user: User | null = null;
  private unsubscribe$ = new Subject<void>();
  isMobile = window.innerWidth <= 600;

  menu: MenuElement[] = [];

  constructor(
    private translateService: TranslateService,
    private config: PrimeNGConfig,
    private guiService: GuiService,
    private auth: AuthService
  ) {
    this.guiService.isMobile$
      .pipe(takeUntil(this.unsubscribe$))
      .subscribe(isMobile => {
        this.isMobile = isMobile;
      });

    this.auth.user$.pipe(takeUntil(this.unsubscribe$)).subscribe(async user => {
      this.user = user;
      await this.initMenuLists();
    });
  }

  async ngOnInit() {
    await this.initMenuLists();
    await this.loadLang();
  }

  ngOnDestroy() {
    this.unsubscribe$.next();
    this.unsubscribe$.complete();
  }

  async login() {
    await this.auth.login();
  }

  async initMenuLists() {
    await this.initMenuList();
    await this.initLangMenuList();
    await this.initUserMenuList();
  }

  async initMenuList() {
    this.menu = [
      {
        label: 'common.news',
        routerLink: ['/'],
        icon: 'pi pi-home',
      },
      {
        label: 'header.menu.about',
        routerLink: ['/about'],
        icon: 'pi pi-info',
      },
    ];

    if (this.auth.user$.value) {
      this.menu.push({
        label: 'header.menu.admin',
        routerLink: ['/admin'],
        icon: 'pi pi-cog',
        visible: await this.auth.isAdmin(),
      });
    }
  }

  async initLangMenuList() {
    this.langList = [
      {
        label: 'English',
        command: () => {
          this.translate('en');
          this.setLang('en');
        },
      },
      {
        label: 'Deutsch',
        command: () => {
          this.translate('de');
          this.setLang('de');
        },
      },
    ];
  }

  async initUserMenuList() {
    this.userMenuList = [
      {
        label: `${this.user?.username}`,
        disabled: true,
        visible: !!this.user,
      },
      {
        separator: true,
      },
      {
        label: this.translateService.instant('header.logout'),
        command: () => {
          this.auth.logout().then(() => {
            console.log('logout');
          });
        },
        icon: 'pi pi-sign-out',
      },
    ];
  }

  translate(lang: string) {
    this.translateService.use(lang);
    this.translateService
      .get('primeng')
      .subscribe(res => this.config.setTranslation(res));
  }

  async loadLang() {
    const lang = 'en';
    this.setLang(lang);
    this.translate(lang);
  }

  setLang(lang: string) {
    // this.settings.setSetting(`lang`, lang);
    console.log('setLang', lang);
  }
}
